
#include <AD9833.h>  // Function Generator Library
#include <U8g2lib.h> //u8g2 library for OLED display
#include <Wire.h>    // library for I2C communication

U8G2_SSD1306_128X64_NONAME_F_HW_I2C u8g2(U8G2_R0, /* reset=*/ U8X8_PIN_NONE); // initialization for the OLED display
//============================================================================
// Define rotary encoder pins
#define CLK 2           // Interrupt Pin 0
#define DT 3            // Interrupt Pin 1
#define SW 8            // Rotary Encoder Push Button Pin

// Define push button pins
#define leftButton 12   // Left Push Button 
#define rightButton 11  // Right Push Button

// Defining Waves
#define SINE 1          // Wave 1
#define TRIANGLE 4      // Wave 2
#define SQUARE 2        // Wave 3
//============================================================================
// Define images (Made using Lopaka.app)

// Triangle Wave Image
static const unsigned char triangleWave[] U8X8_PROGMEM = {0x00,0x01,0x40,0x00,0x10,0x00,0x04,0x80,0x03,0xe0,0x00,0x38
,0x00,0x06,0xc0,0x06,0xb0,0x01,0x6c,0x00,0x03,0x60,0x0c,0x18,0x03,0xc6,0x80,0x01,0x30,0x18,0x0c,0x06,0x83,0xc1,0x00
,0x18,0x30,0x06,0x8c,0x01,0x63,0x00,0x0c,0x60,0x03,0xd8,0x00,0x36,0x00,0x06,0xc0,0x01,0x70,0x00,0x1c,0x00,0x03,0x80
,0x00,0x20,0x00,0x08,0x00};

// Square Wave Image
static const unsigned char squareWave[] U8X8_PROGMEM = {0xf8,0x3f,0x80,0xff,0x03,0xf8,0x07,0x08,0x20,0x80,0x00,0x02
,0x08,0x00,0x08,0x20,0x80,0x00,0x02,0x08,0x00,0x08,0x20,0x80,0x00,0x02,0x08,0x00,0x08,0x20,0x80,0x00,0x02,0x08,0x00
,0x08,0x20,0x80,0x00,0x02,0x08,0x00,0x08,0x20,0x80,0x00,0x02,0x08,0x00,0x08,0x20,0x80,0x00,0x02,0x08,0x00,0x0f,0xe0
,0xff,0x00,0xfe,0x0f,0x00};

// Sine Wave Image (Use either Higher Frequency or Lower Frequency Sine Wave, Remove the other Sine Wave)
static const unsigned char sineWave[] U8X8_PROGMEM = {0x38,0x00,0x0e,0x80,0x03,0xe0,0x00,0x44,0x00,0x11,0x40,0x04,0x10
,0x01,0x82,0x80,0x20,0x20,0x08,0x08,0x02,0x82,0x80,0x20,0x20,0x08,0x08,0x02,0x83,0xc1,0x60,0x30,0x18,0x0c,0x06,0x01
,0x41,0x40,0x10,0x10,0x04,0x04,0x00,0x41,0x40,0x10,0x10,0x04,0x04,0x00,0x22,0x80,0x08,0x20,0x02,0x00,0x00,0x1c,0x00
,0x07,0xc0,0x01,0x00}; // Higher frequency sine

/*
static const unsigned char sineWave[] U8X8_PROGMEM = {0xe0,0x00,0x00,0x0e,0x00,0xe0,0x00,0x18,0x03,0x80,0x31,0x00,0x18
,0x03,0x04,0x04,0x40,0x40,0x00,0x04,0x04,0x04,0x04,0x40,0x40,0x00,0x04,0x04,0x02,0x08,0x20,0x80,0x00,0x02,0x00,0x01
,0x10,0x10,0x00,0x01,0x01,0x00,0x01,0x10,0x10,0x00,0x01,0x01,0x00,0x00,0x60,0x0c,0x00,0xc6,0x00,0x00,0x00,0x80,0x03
,0x00,0x38,0x00,0x00}; // Lower frequency sine
*/

// Pointer Arrow Image 
static const unsigned char arrow[] U8X8_PROGMEM = {0x3f,0x1e,0x0c};

//============================================================================
// Initialize variables 
int prevDir, currDir;     // Previous and Current Readings of the CLK Pin of the Rotary Encoder
long debounceTime = 300;  // Debounce time for Push Buttons in milliseconds
long currentUp;           // Time in milliseconds when the last push button was pressed

volatile unsigned long frequency = 10000; // Setting the initial frequency

long digit = 1000;   // Setting initial position of the Pointer Arrow

int wave = 0;        // Setting the output off initally

AD9833 AD(6, 5, 4);   // Define AD9833 Pins 
//============================================================================
void setup() {
  pinMode(CLK, INPUT);
  pinMode(DT, INPUT);
  pinMode(SW, INPUT_PULLUP);
  pinMode(leftButton, INPUT_PULLUP);
  pinMode(rightButton, INPUT_PULLUP);

  prevDir = digitalRead(CLK);

  u8g2.begin(); // Start the u8g2 library
  u8g2.setFontMode(1);
  u8g2.setBitmapMode(1);
  u8g2.setFont(u8g2_font_profont15_tr); // Select font

  //SPI.begin(); 

  AD.begin();  

  displayOLED(frequency, digit, wave);
  AD.setWave(0);

  attachInterrupt(digitalPinToInterrupt(2), rotaryInterrupt, CHANGE); // Set CLK as interrupt pin 0
  attachInterrupt(digitalPinToInterrupt(3), rotaryInterrupt, CHANGE); // Set DT as interrupt pin 1
}
//============================================================================
// Main Loop
void loop() {
  digit = updateDigit(digit);

  wave = updateWave(wave);

  displayOLED(frequency, digit, wave);
  generateWave(frequency, wave);
 // delay(10);  
}
//============================================================================
// Interrupt ISR to update frequency
void rotaryInterrupt(void){
  frequency = updateFrequency(frequency, digit);
  return;
}
//============================================================================
// Send wave and frequency data to AD9833 IC
void generateWave(long frequency, int wave){
  if (wave == 1){
    AD.setWave(SINE);
  }
  else if (wave == 2){
    AD.setWave(TRIANGLE);
  }
  else if (wave == 3){
    AD.setWave(SQUARE);
  }
  else{
    AD.setWave(0);
  }
  AD.setFrequency(frequency, 0);
}
//============================================================================
// Update Wave Function
int updateWave(int wave){
  if (button(SW) == 1){
    if (wave < 3){
      return wave + 1;
    }
    else {
      return 0;
    }
  }
  else{
   return wave;
  }
}
//============================================================================
// Update OLED Display
void displayOLED(long frequency, long digit, int wave){
  int i = 0;
  char buffer[10];
  while (digit != 0){
    digit = (long)(digit / 10);
    i++;
  }
  u8g2.clearBuffer();

  frequencyToString(frequency, buffer);

  u8g2.setFont(u8g2_font_profont15_tr); // Font size for Wave Selection

  if (wave == 1){ // Sine Wave
  u8g2.drawStr(4, 11, "Sine");              
  u8g2.drawXBMP(70, 3, 51, 9, sineWave);    
  }
  else if(wave == 2){ // Triangle Wave
  u8g2.drawStr(4, 11, "Triangle");             
  u8g2.drawXBMP(70, 3, 51, 9, triangleWave); 
  }
  else if(wave == 3){ // Square Wave
  u8g2.drawStr(4, 11, "Square");              
  u8g2.drawXBMP(70, 3, 51, 9, squareWave); 
  }

  u8g2.drawStr(4, 28, "Frequency:");           

  u8g2.drawXBMP(102 - i * 12, 35, 6, 3, arrow);  

  if (wave == 0){
    u8g2.drawStr(4, 11, "Output OFF");
    u8g2.drawLine(77, 7, 124, 7);
    u8g2.setFont(u8g2_font_profont17_tr);
    u8g2.drawStr(100, 29, "OFF");
  }
  else{
    u8g2.setFont(u8g2_font_profont17_tr);
    u8g2.drawStr(100, 29, "ON");
  }

  u8g2.setFont(u8g2_font_profont22_tr);
  u8g2.drawStr(4, 55, buffer);     
  u8g2.drawStr(103, 55, "Hz");
  
  u8g2.sendBuffer();	
}
//============================================================================
// Convert Frequency to String for the OLED Display
void frequencyToString(long frequency, char buffer[]){
  if ((long)(frequency / 10) == 0){
    sprintf(buffer, "       %ld", frequency);
    return;
  }
  else if ((long)(frequency / 100) == 0){
    sprintf(buffer, "      %ld", frequency);
    return;
  }
  else if ((long)(frequency / 1000) == 0){
    sprintf(buffer, "     %ld", frequency);
    return;
  }
  else if ((long)(frequency / 10000) == 0){
    sprintf(buffer, "    %ld", frequency);
    return;
  }
  else if ((long)(frequency / 100000) == 0){
    sprintf(buffer, "   %ld", frequency);
    return;
  }
  else if ((long)(frequency / 1000000) == 0){
    sprintf(buffer, "  %ld", frequency);
    return;
  }
  else if ((long)(frequency / 10000000) == 0){
    sprintf(buffer, " %ld", frequency);
    return;
  }
  else{
    sprintf(buffer, "%ld", frequency);
    return;
  }
}
//============================================================================
// Get rotary encoder input
int rotaryEncoder(){
  currDir = digitalRead(CLK);
  if (prevDir != currDir){
    if(digitalRead(DT) != currDir){  // Clockwise
      prevDir = currDir; 
      return 1;
    }
    else{                            // Counter-Clockwise
      prevDir = currDir;
      return -1;
    }
  }
  prevDir = currDir;
  return 0;
}
//============================================================================
// Get push button input
int button(int button){
  if (digitalRead(button) == LOW && (millis() - currentUp > debounceTime)){
    currentUp = millis();
    return 1;
  }
  else{
    return 0;
  }
}
//============================================================================
// Update pointer arrow's position
long updateDigit(long n){
  if (button(leftButton) == HIGH && n < 1000000){
    return n * 10;
  }
  else if(button(rightButton) == HIGH && n > 1){
    return n / 10;
  }
  else{
    return n;
  }
}
//============================================================================
// Update frequency
long updateFrequency(long n, long mult){
  int x = rotaryEncoder();
  long f;
  if (x == 1){
    f = n + mult;
    if (f <= 10000000)
      return f;
    else
      return n;
  }
  else if (x == -1){
    f = n - mult;
    if (f >= 0)
      return f;
    else 
      return n;
  }
  else{
    return n;
  }
}
//============================================================================


